(function() {
    var editorCfg = {
        "name": {
            "title": "修改姓名",
            "apiKey": "empName",
            "metaKey": "userName",
            "components": [{
                "type": "input",
                "placeholder": "请输入姓名",
                "maxlength": "10",
                "checkValue":function(val,self){
                    if(val==""){
                        self.errorText="请输入姓名！";
                    }else{
                        self.errorText="";
                    }
                }
            }],
            getValue: function() {
                var cfg = editorCfg[this.paras.key];
                var val = this.$.find("input,textarea").val();
                var resultName = /^[\u4e00-\u9fa5_a-zA-Z0-9_\(\)\（\）]+$/
                if (val == '') {
                    this.errorText = "请输入姓名！";
                } else if (resultName.test(val)) {
                    this.errorText = "";
                } else {
                    this.errorText = "姓名只能输入中英文加数字或括号";
                }
                var ret = {};
                ret[cfg.apiKey] = val;
                return ret;
            },
        },
        "nickname": {
            "title": "修改昵称",
            "apiKey": "nickname", //mgjnickname
            "metaKey": "nickname",
            "tip": "注：您的昵称将在预约系统显示，发消息给顾客时，也将以昵称署名",
            "components": [{
                "type": "input",
                "placeholder": "请输入昵称",
                "maxlength": "20",
                "checkValue":function(val,self){
                    if(val==""){
                        self.errorText="请输入昵称！";
                    }else{
                        self.errorText="";
                    }
                }
            }],

        },
        "empCode": {
            "title": "绑定员工码",
            "apiKey": "empCode", //mgjnickname
            "metaKey": "empCode",
            "tip": "注：请输入员工开单号码",
            "components": [{
                "type": "number",
                "placeholder": "请输入员工码",
                "maxlength": "3",
                "checkValue":function(val,self){
                    if(val==""){
                        self.errorText="请输入员工码！";
                    }else{
                        self.errorText="";
                    }
                }
            }],
            "saveAction": function(option) {
                var empCode = option.empCode;
                if (!empCode || empCode.length != 3 || !amGloble.isNumber(empCode)) {
                    amGloble.msg('请输入三位数字的员工码！');
                    return;
                }
                amGloble.loading.show("正在提交，请稍候...");
                var userInfo = amGloble.metadata.userInfo;
                amGloble.api.bindEmpCode.exec({
                    mobile: userInfo.mobile,
                    empCode: option.empCode,
                    shopId: userInfo.shopId,
                    parentShopId: userInfo.parentShopId
                }, function(ret) {
                    amGloble.loading.hide();
                    if (ret.code === 0) {
                        if (amGloble.metadata.userInfo.empCode) {
                            amGloble.msg('员工码修改成功！')
                        } else {
                            amGloble.msg('员工码保存成功！')
                        }
                        amGloble.metadata.userInfo.empCode = option.empCode;
                        var metaKey = amGloble.getMetaStorageKey && amGloble.getMetaStorageKey();
                        if (metaKey) {
                            var cacheDataString = localStorage.getItem(metaKey);
                            var cacheData = JSON.parse(cacheDataString);
                            cacheData.userInfo.empCode = option.empCode;
                            localStorage.setItem(metaKey, JSON.stringify(cacheData))
                        }
                        $.am.page.back();
                    } else {
                        amGloble.msg(ret.message || "提交失败，请重试！", true);
                    }
                })
            },
        },
        "price": {
            "title": "修改剪发价",
            "apiKey": "mgjhaircutprice",
            "metaKey": "cutPrice",
            "tip": "注：顾客预约时可以看到你设置的剪发价",
            "components": [{
                "type": "input",
                "placeholder": "请输入剪发价",
                "maxlength": "7"
            }]
        },
        "tags": {
            "title": "修改标签",
            "apiKey": "mgjtags",
            "metaKey": "tags",
            "tip": "注：顾客预约时可以看到你设置的标签",
            "imgTip": true,
            "components": [{
                "type": "input",
                "title": "标签1",
                "placeholder": "请输入",
                "maxlength": "10"
            }, {
                "type": "input",
                "title": "标签2",
                "placeholder": "请输入",
                "maxlength": "10"
            }, {
                "type": "input",
                "title": "标签3",
                "placeholder": "请输入",
                "maxlength": "10"
            }, {
                "type": "input",
                "title": "标签4",
                "placeholder": "请输入",
                "maxlength": "10"
            }],
            getValue: function() {
                // console.log("自定义getValue");
                var cfg = editorCfg[this.paras.key];
                var ret = {};
                var values = [];
                this.$.find("input,textarea").each(function(i, item) {
                    var value = $.trim($(item).val());
                    if (value) {
                        values.push(value)
                    }
                });
                ret[cfg.apiKey] = values.join(",");
                return ret;
            },
            setValue: function() {
                // console.log("自定义setValue");
                var cValue = this.paras.cValue;
                if (cValue) {
                    var values = cValue.split(",");
                    this.$.find("input,textarea").each(function(i, item) {
                        var value = values[i];
                        if (value) {
                            $(item).val(value);
                        }
                    });
                }
            }
        },
        "description": {
            "title": "修改简介",
            "apiKey": "mgjbio",
            "metaKey": "bio",
            "components": [{
                "type": "textarea",
                "placeholder": "请输入简介"
            }]
        },
        comment:{
            "title": "评论",
            "apiKey": "submitComment",
            "metaKey": "comment",
            "components": [{
                "type": "textarea",
                "maxlength": "100",
                "placeholder": "请输入评论内容",
                "checkValue":function(val,self){
                    if(val==""){
                        self.errorText="请输入评论内容！";
                    }else{
                        self.errorText="";
                    }
                }
            }]
        },
        complain:{
            "title": "投诉",
            "apiKey": "submitComplaint",
            "metaKey": "complain",
            "components": [{
                "valueKey":"contact",
                "type": "input",
                "maxlength": "11",
                "placeholder": "请输入联系电话",
                "checkValue":function(val,self){
                    if(!amGloble.rePhone.test(val)){
                        self.errorText="请输入正确的手机号！";
                    }else{
                        self.errorText="";
                    }
                }
            },{
                "valueKey":"comment",
                "type": "textarea",
                "maxlength": "100",
                "placeholder": "请输入投诉内容",
                "checkValue":function(val,self){
                    if(val==""){
                        self.errorText="请输入投诉内容！";
                    }else{
                        self.errorText="";
                    }
                }
            }],
            getValue:function(){
                var cfg = editorCfg[this.paras.key];
                var ret = {};
                this.$.find("input,textarea").each(function(i, item) {
                    var value = $.trim($(item).val());
                    var key=$(item).parents("li").data("key");
                    ret[key]=value;
                });
                return ret;
            },
            setValue: function() {
                // console.log("自定义setValue");
                var cValue = this.paras.cValue;
                if (cValue) {
                    var values = cValue.split(",");
                    this.$.find("input,textarea").each(function(i, item) {
                        var value = values[i];
                        if (value) {
                            $(item).val(value);
                        }
                    });
                }
            }
        },
        "mobile":{
            "title": "修改手机号",
            "apiKey": "mobile",
            "metaKey": "mobile",
            "components": [{
                "type": "input",
                "placeholder": "请输入手机号",
                "maxlength": "11",
                "checkValue":function(val,self){
                    if(val==""){
                        self.errorText="请输入手机号！";
                    }else{
                        self.errorText="";
                    }
                }
            }],
        },
        "idNo":{
            "title": "修改身份证号",
            "apiKey": "idNo",
            "metaKey": "idNo",
            "components": [{
                "type": "input",
                "placeholder": "请输入身份证号",
                "checkValue":function(val,self){
                    if(val==""){
                        self.errorText="请输入身份证号！";
                    }else{
                        self.errorText="";
                    }
                }
            }],
        },
        "userName":{
            "title": "修改用户名",
            "apiKey": "userName",
            "metaKey": "userName",
            "components": [{
                "type": "input",
                "placeholder": "请输入用户名",
                "maxlength": "15",
                "checkValue":function(val,self){
                    if(val==""){
                        self.errorText="请输入用户名！";
                    }else{
                        self.errorText="";
                    }
                }
            }],
        },
        "passWord":{
            "title": "修改密码",
            "apiKey": "passWord",
            "metaKey": "passWord",
            "components": [{
                "type": "password",
                "placeholder": "请输入密码",
                "maxlength": "15",
                "checkValue":function(val,self){
                    if(val==""){
                        self.errorText="请输入密码！";
                    }else{
                        self.errorText="";
                    }
                }
            }],
        },
    };

    var chtml = {
        "input": '<input type="text">',
        "textarea": '<textarea></textarea>',
        "number": '<input type="number">',
        "password": '<input type="password">',
    };


    var self = amGloble.page.userinfoEditor = new $.am.Page({
        id: "page-userinfoEditor",
        init: function() {
            var self = this;
            this.errorText="";
            this.$save = this.$header.children(".right").vclick(function() {
                if(self.paras.save){
                    self.paras.save.call(self,editorCfg);
                }else{
                    self.save();
                }

            });
            this.$title = this.$header.find("p");

            this.$ul = this.$.find(".listArea");

            this.$tip = this.$.find(".tip");
            this.$imgTip = this.$.find(".imgTip");
        },
        beforeShow: function(paras) {

            var $curpay = amGloble.metadata&&amGloble.metadata.configs.v4_barberfajiaList;
            //console.log(editorCfg.price.components.placeholder);
            if($curpay) {
                editorCfg.price.title = '修改' + $curpay;
                editorCfg.price.tip = '注：顾客预约时可以看到你设置的'+$curpay;
                editorCfg.price.components[0].placeholder='请输入' + $curpay;
            }
           //console.log(editorCfg.price.title);
            this.paras = paras;
            this.render();
        },
        afterShow: function(paras) {
            this.$.find("input,textarea").eq(0).focus();
        },
        beforeHide: function() {},
        render: function(key) {
            var self=this;
            var cfg = editorCfg[this.paras.key];

            this.$title.text(cfg.title);
            var $ul = this.$ul.empty();
            for (var i = 0; i < cfg.components.length; i++) {
                var item = cfg.components[i];
                $li = $('<li></li>');
                if (item.title) {
                    $li.append('<span class="title">' + item.title + '</span>');
                } else {
                    $li.addClass("noTitle");
                }
                if(item.valueKey){
                    $li.data("key", item.valueKey || "");
                }
                $li.append('<span class="input">' + chtml[item.type] + '</span>');
                $li.find("input,textarea").attr("placeholder", item.placeholder || "");
                $li.find("input,textarea").attr("maxlength", item.maxlength || "");
                $ul.append($li);
                if(item.checkValue){
                    $li.on("checkValue",function(){
                        item.checkValue($li.find("input,textarea").val(),self);
                    }).attr("ischeck",true);
                }
            }

            console.log(cfg);

            if (cfg.tip) {
                this.$tip.html(cfg.tip).show();
            } else {
                this.$tip.hide();
            }
            if (cfg.imgTip) {
                this.$imgTip.show();
            } else {
                this.$imgTip.hide();
            }

            if (cfg.setValue) {
                cfg.setValue.call(this);
            } else {
                this.setValue();
            }
        },
        setValue: function() {
            var cfg = editorCfg[this.paras.key];
            this.$.find("input,textarea").val(this.paras.cValue);
        },
        getValue: function() {
            var cfg = editorCfg[this.paras.key];
            var ret = {};
            ret[cfg.apiKey] = this.$.find("input,textarea").val();
            return ret;
        },
        save: function() {
            var user = amGloble.metadata.userInfo;
            var v;
            var cfg = editorCfg[this.paras.key];
            if (cfg.getValue) {
                option = cfg.getValue.call(this);
            } else {
                option = this.getValue();
            }

            option.userid = user.userId;
            option.userType = user.userType;
            option.shopid = user.shopId;

            console.log(option);
            //return;

            if(this.errorText){
                amGloble.msg(this.errorText);
                return;
            }
            if (cfg.saveAction) {
                cfg.saveAction.call(this, option);
                return;
            }

            amGloble.loading.show("正在提交，请稍候...");

            amGloble.api.userModify.exec(option, function(ret) {
                amGloble.loading.hide();
                console.log(ret);
                if (ret.code == 0) {
                    amGloble.msg("修改成功！");
                    user[cfg.metaKey] = option[cfg.apiKey];
                    var userInfoList = amGloble.metadata.userInfoList || [];
                    userInfoList.forEach(function(item) {
                        if (item.userId == user.userId) {
                            item[cfg.metaKey] = option[cfg.apiKey]
                        }
                    })
                    var metaKey = amGloble.getMetaStorageKey();
                    var json = localStorage.getItem(metaKey);
                    try{
                        if(json){
                            json = JSON.parse(json);
                        }
                    }catch(e){
                        console.error(e);
                    }
                    if(json){
                        json.userInfo = user;
                        json.userInfoList = userInfoList;
                        localStorage.setItem(metaKey,JSON.stringify(json));
                    }
                    //调用一次metadata，刷新缓存

                    amGloble.api.metadata.exec({
                        parentShopId: config.parentShopId,
                        token: config.token
                    }, function(ret, isLocal) {
                        console.log("metadata", ret, isLocal);
                    });
                    $.am.page.back("", { userInfoUpdate: { changeUserInfo: true } });
                } else {
                    amGloble.msg(ret.message || "提交失败，请重试！", true);
                }
            });
        }
    });

})();
